import { NextResponse, NextRequest } from 'next/server';

/**
 * Enumeration representing common HTTP status codes and their numeric values.
 */
declare enum HttpStatusEnum {
    OK = 200,
    CREATED = 201,
    ACCEPTED = 202,
    NO_CONTENT = 204,
    BAD_REQUEST = 400,
    UNAUTHORIZED = 401,
    FORBIDDEN = 403,
    NOT_FOUND = 404,
    INTERNAL_SERVER_ERROR = 500,
    SERVICE_UNAVAILABLE = 503
}
/**
 * Utility class to provide additional functionality around HTTP status codes.
 */
declare class HttpStatus {
    static OK: HttpStatusEnum;
    static CREATED: HttpStatusEnum;
    static ACCEPTED: HttpStatusEnum;
    static NO_CONTENT: HttpStatusEnum;
    static BAD_REQUEST: HttpStatusEnum;
    static UNAUTHORIZED: HttpStatusEnum;
    static FORBIDDEN: HttpStatusEnum;
    static NOT_FOUND: HttpStatusEnum;
    static INTERNAL_SERVER_ERROR: HttpStatusEnum;
    static SERVICE_UNAVAILABLE: HttpStatusEnum;
    /**
     * Retrieves the textual description of a given HTTP status code.
     * @param status The HTTP status code.
     * @returns The textual description of the status code, or "Unknown Status" if not found.
     */
    static getStatusText(status: HttpStatusEnum): string;
    /**
     * Validates and converts a numeric HTTP status code to the corresponding `HttpStatusEnum` value.
     * @param statusCode The numeric HTTP status code.
     * @returns The corresponding `HttpStatusEnum` value if valid, or `undefined` otherwise.
     */
    static fromCode(statusCode: number): HttpStatusEnum | undefined;
    /**
     * Retrieves both the numeric HTTP status code and its textual description.
     * @param statusCode The numeric HTTP status code.
     * @returns An object containing the status code and its description, or `undefined` if the code is invalid.
     */
    static getStatusWithText(statusCode: number): {
        status: HttpStatusEnum;
        text: string;
    } | undefined;
}

/**
 * Custom error class to standardize error handling in controllers.
 * Includes a unique key, detailed message, and associated HTTP status code.
 */
declare class ControllerError extends Error {
    key: string;
    message: string;
    httpStatus: number;
    /**
     * Constructor for the `ControllerError` class.
     * @param key A unique key identifying the error.
     * @param message An optional detailed message for the error. If not provided, a default message is generated using the key.
     * @param httpStatus The HTTP status code for the error (default is 400 - Bad Request).
     */
    constructor(key: string, message?: string, httpStatus?: number);
}

/**
 * Builds a standard HTTP response for API routes.
 * @param data The response data, which can be any valid response body.
 * @param response The response metadata, including HTTP status code and optional headers.
 * @param contentType The content type of the response, default is "application/json".
 * @returns A Next.js `NextResponse` object with the formatted body and headers.
 */
declare function buildResponse({ data, response, contentType, }: {
    data: unknown;
    response: {
        httpStatus: number;
        headers?: Record<string, string>;
    };
    contentType?: string;
}): NextResponse<unknown>;

declare class CustomHeaders extends Headers {
    private request;
    constructor(request: NextRequest);
    getBearerToken(): string;
    /**
     * Parses and returns cookies as a key-value object.
     * @returns An object representing the cookies.
     */
    getCookies(): Record<string, string>;
}
type ControllerContext<T> = {
    request: NextRequest;
    context?: {
        params?: Record<string, string>;
        searchParams?: Record<string, string>;
    };
    body?: T;
    headers: CustomHeaders;
};
interface HandlerResponse {
    data?: unknown;
    response: {
        httpStatus: number;
    };
}
/**
 * Wrapper function to handle API route logic with error handling and context preparation.
 * @param handler The function that handles the request logic.
 * @returns A handler for Next.js API routes.
 */
declare function controller<T extends HandlerResponse | NextResponse>(handler: (ctx: ControllerContext<T>) => Promise<HandlerResponse | NextResponse>): {
    (request: NextRequest, context: {
        params: Promise<any>;
    }): Promise<Response>;
    authorize(authorizationCheck: (token: string) => boolean | Promise<boolean>): (request: NextRequest, context: {
        params: Promise<any>;
    }) => Promise<Response>;
};

export { ControllerError, HttpStatus, buildResponse, controller };
